#!/usr/bin/perl -W 

#######################################################################
# Stimuli Order Randomisation 
#
# author: Gerhard Kremer, 2010 
#
# usage: 
# ./rep-pick-random-lines.perl [options] 
#
# purpose: 
# A script to be used repeatedly to produce random subsets of stimuli
# with the aim of having a quite balanced distribution of stimuli
# frequencies used in one experiment. If there is a file of stimuli
# not used in the previous run of the script, it is used first to pick
# stimuli from there (filename: stimulifilename_remrand.txt).
#######################################################################

use strict;
# consider use Math::Random or better Math::TrulyRandom
use File::Temp qw/ tempfile /; 
use Getopt::Long;
# Try Getopt::Declare for intermixed usage comments

# set random seed once per call
# beware when calling this script from within a loop (don't)
srand(time|$$);

# my defaults
my $num=333;
my $winlf=''; # False
my $outfilenametemplate="exp-satzset";
my $subjectsfile="subjectsfile_judgemexp.txt";
my $subjectname="";
my $stimfile="stimuli-phrases.txt";
my $remrandfile=$stimfile; 
if($stimfile=~/\.txt/){
    $remrandfile=~s/\.txt/_remrand\.txt/;
}else{
    $remrandfile=~s/^/remrand_/;
};

our ($outfhandle, $outfname, $remrandnum, @remstims, %remains, @stimuliset,
     $stimulus, $count, $choice);

# print usage information
sub usage() {
    print STDERR << "EOU";
USAGE
$0 [Options]
    
PURPOSE
    A script to be used repeatedly to produce random subsets of stimuli
    with the aim of having a quite balanced distribution of stimuli 
    frequencies used in one experiment. If there is a file of stimuli not
    used in the previous run of the script, it is used first to pick
    stimuli from there (filename: stimulifilename_remrand.txt).
    
OPTIONS
    --num
    --number <n>	number of stimuli to randomly spit out 
    			(Default: $num)
    --st
    --stimfile <file>	file with complete stimuli set
    			(Default: $stimfile)
    --sb
    --subfile <file>	subjectsfile: saves outfilename and subjectname
    			(Default: $subjectsfile)
    --name <"A Name">	the current subject name - multiple word in "..."
    			(will be used in the subjectsfile; obligatory)
    -t
    --template <pref>	template prefix for outfile
    			(Default: $outfilenametemplate)
    -w			convert style of linefeeds (Unix to Windows)
EOU
exit;
};

GetOptions('number|num:i'=> \$num,
	   'stimfile:s'=>\$stimfile,
	   'subfile:s'=>\$subjectsfile,
	   'name=s'=> \$subjectname,
	   'template:s'=>\$outfilenametemplate,
	   'w'=>\$winlf,
	   'help'=> \&usage)
    or die "Options parsing error\n";

# subject name is required
if($subjectname eq ""){
    die "please specify a subject name\nSee $0 -h for details\n"}; 

# read stimulusfile
open(STIMULI,$stimfile) || die "not an existing file:$stimfile \n";
@stimuliset=<STIMULI>;
close STIMULI;

# sanity check: enough stimuli ins set for requested number?
die "more stimuli requested than in stimulifile -- some would be doubled\n"
if $num > @stimuliset;

# use tempfile functionality to create unique filenames
($outfhandle,$outfname)=tempfile("$outfilenametemplate"."XXXX",
				 SUFFIX => ".txt",UNLINK=>0);


# init
$remrandnum=0;
# read remain-file if it exists from previous run
if (-f $remrandfile){
    open(REMRAND,"$remrandfile") || die "not an existing file:$stimfile \n";
    @remstims=<REMRAND>;
    close REMRAND;
    foreach my $remstim (@remstims){
	$remains{$remstim}=1;
    };
    $remrandnum=scalar (keys %remains);
};

# are there enough stimuli left in remain-file?
# then select them for random picks 
# (overwriting stimuliset array with this stimuli subset)
if($remrandnum>=$num){
    @stimuliset=@remstims;
}
#... otherwise print them all and select more from whole stimuli set
elsif($remrandnum<$num){
    $num-=$remrandnum;
    print $outfhandle join("\n",@remstims),"\n";
    open(ALLSTIMULI,"$stimfile") || die "not an existing file:$stimfile \n";;
    @stimuliset = <ALLSTIMULI>;
    close ALLSTIMULI;
#    unlink $remrandfile;
};

# randomly select stimuli from current stimuli set
$count=0;
while ( @stimuliset && $count <= $num-1) {
    $choice = rand @stimuliset;
    if($remrandnum<$num && exists($remains{$stimuliset[$choice]})){
	next};
    $stimulus = splice(@stimuliset, $choice, 1); 
    print $outfhandle "$stimulus\n"; 
    $count++;
}

# convert outfile to Windows/DOS format
my $convertcall=q[perl -i -pe 's/\r\n|\n|\r/\r\n/g']." $outfname";
if($winlf){
    if(system($convertcall) != 0){
	die "ERRORS: $?\n"; 
	unlink $outfname
	};
};

# print the now remaining stimuli into remain-file
# just don't write if @stimuliset is now empty? 
#... or is algorithm working anyway?
open(REMRANDOUT,"> $remrandfile");
print REMRANDOUT @stimuliset;
close REMRANDOUT;

# print outfilename used and subject (name or no.)
open(SUBJECTS,">> $subjectsfile"); 
print SUBJECTS "$outfname\t$subjectname\n"; 
close SUBJECTS; 

